/*****************************************************************************
 *****************************************************************************
Copyright (c) 2000-2001 Intel Corporation. 
 
All rights reserved.
Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
 
 3. Neither the name of Intel Corporation nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.
 
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 *****************************************************************************
 ****************************************************************************/

/**********************************************************************
 *                                                                     *
 * INTEL CORPORATION                                                   *
 *                                                                     *
 * This software is supplied under the terms of the license included   *
 * above.  All use of this software must be in accordance with the     *
 * terms of that license.                                              *
 *                                                                     *
 * Module Name:  idiag_e1000.h                                         *
 *                                                                     *
 * Abstract:     Header file for data structure definition for common  *
 *               diagnostics support                                   *
 *                                                                     *
 **********************************************************************/

#ifndef _IDIAG_E1000_H
#define _IDIAG_E1000_H

/* Unique base driver identifier */

#define IDIAG_E1000_DRIVER				0x02

/* e1000 diagnostic commands */

#define IDIAG_E1000_DIAG_REG_TEST		0x01
#define IDIAG_E1000_DIAG_FIFO_TEST		0x02
#define IDIAG_E1000_DIAG_XSUM_TEST		0x03
#define IDIAG_E1000_DIAG_INTR_TEST		0x04
#define IDIAG_E1000_DIAG_LOOPBACK_TEST	0x05
#define IDIAG_E1000_DIAG_LINK_TEST		0x06

typedef struct {
	unsigned short Reg;
	unsigned short Pad;
	unsigned long WriteValue;
	unsigned long ReadValue;
} idiag_e1000_diag_reg_test_param_t;

typedef struct {
	unsigned short Offset;
	unsigned short Pad;
	unsigned long WriteValue;
	unsigned long ReadValue;
} idiag_e1000_diag_fifo_test_param_t;

typedef struct {
	unsigned long ExpectedChecksum;
	unsigned long ActualChecksum;
} idiag_e1000_diag_eeprom_test_param_t;

typedef enum {
	IDIAG_E1000_INTR_TEST_OK,
	IDIAG_E1000_INTR_TEST_NOT_EXEC,
	IDIAG_E1000_INTR_TEST_FAILED_WHILE_DISABLED,
	IDIAG_E1000_INTR_TEST_FAILED_WHILE_ENABLED,
	IDIAG_E1000_INTR_TEST_FAILED_MASKED_ENABLED
} idiag_e1000_diag_intr_test_param_t;

#define IDIAG_E1000_DIAG_MAC_LB		0x01
#define IDIAG_E1000_DIAG_TCVR_LB	0x02
#define IDIAG_E1000_DIAG_EXT_LB		0x04

#define IDIAG_E1000_ALL_LB		(IDIAG_E1000_DIAG_MAC_LB  | \
				                 IDIAG_E1000_DIAG_TCVR_LB | \
								 IDIAG_E1000_DIAG_EXT_LB)

#define IDIAG_E1000_DEFAULT_LB	(IDIAG_E1000_DIAG_MAC_LB  | \
				                 IDIAG_E1000_DIAG_TCVR_LB)

typedef enum {
	IDIAG_E1000_LOOPBACK_TEST_OK,
	IDIAG_E1000_LOOPBACK_TEST_NOT_EXEC,
	IDIAG_E1000_LOOPBACK_TEST_FAILED
} idiag_e1000_diag_loopback_result_t;

typedef struct {
	/* input - loopback modes to run (MAC, TCVR, EXT) */
	unsigned int mode;

	/* results */
	idiag_e1000_diag_loopback_result_t mac;
	idiag_e1000_diag_loopback_result_t tcvr;
	idiag_e1000_diag_loopback_result_t ext;
} idiag_e1000_diag_loopback_test_param_t;

typedef enum {
	IDIAG_E1000_LINK_TEST_UP,
	IDIAG_E1000_LINK_TEST_DOWN,
	IDIAG_E1000_LINK_TEST_NOT_EXEC
} idiag_e1000_diag_link_test_param_t;

#endif
