/*****************************************************************************
 *****************************************************************************
 Copyright (c) 1999 - 2001, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
 ****************************************************************************/

 /***************************************************************************
 **                                                                        **
 ** INTEL CORPORATION                                                      **
 **                                                                        **
 ** This software is supplied under the terms of the license included      **
 ** above.  All use of this software must be in accordance with the terms  **
 ** of that license.                                                       **
 **                                                                        **
 **  Module Name:                                                          **
 **    ians_kernel.c                                                       **
 **                                                                        **
 **  Abstract:                                                             **
 **    This module contains all the functions needed for the interface of  **
 **    the iANS module with the kernel.                                    **
 **                                                                        **
 **  Environment:                                                          **
 **    Kernel Mode (linux 2.2.x, 2.4.x)                                    **
 **                                                                        **
 ***************************************************************************/

#define _IANS_MAIN_MODULE_C_

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/malloc.h>
#include <linux/init.h>
#include <linux/malloc.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/rtnetlink.h>
#include <linux/version.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>

#ifdef LINUX_KERNEL_24X
#include <linux/spinlock.h>
#include <linux/if.h>
#endif
#include <linux/param.h>

#include "ians_kcompat.h"
#include "base_comm.h"
#include "incg_open_utils.h"
#include "ians_base.h"
#include "ians_kernel.h"
#include "incg_net.h"
#include "ans_interface.h"
#include "incg_flow.h"
#include "pubDefs.h"
#include "status.h"
#include "incg_gp_mem.h"


static struct net_device *control_g = NULL;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) 
static spinlock_t ians_lock, ians_ind_lock;
#endif

static BOOLEAN KERNEL_VERSION_GREATER_THAN_2_4_0;

// Forward declaration
#define DEST_ADDR_PTR(osMsg, flow)  \
 (flow == TRANSMIT_FLOW ? (MAC_ADDR *)(osMsg->data) : \
                     (MAC_ADDR *)(osMsg->mac.ethernet->h_dest));

#define SRC_ADDR_PTR(osMsg, flow) \
  (flow == TRANSMIT_FLOW ? ((MAC_ADDR *)(osMsg->data) + 1) :\
		       (MAC_ADDR *)(osMsg->mac.ethernet->h_source));

#define LENGTH_TYPE_PTR(osMsg, flow) \
  (flow == TRANSMIT_FLOW ? (unsigned short *)((MAC_ADDR *)(osMsg->data) + 2) :\
		       (unsigned short *)(&(osMsg->mac.ethernet->h_proto)));

static int iansVadapterOpen(struct net_device *dev);

static int iansVadapterStop ( struct net_device *dev );

static int iansVadapterSend (struct sk_buff *skb, struct net_device *dev);

static struct net_device_stats *iansVadapterGetStats ( struct net_device *dev); 

static void iansVadapterSetMulticastList ( struct net_device *dev );

static int iansVadapterChangeMtu( struct net_device *dev, int new_mtu);

static int iansVadapterSetMacAddress(struct net_device *dev, void *addr);

static int iansVadapterDoIoctl ( struct net_device *dev, struct ifreq *rq, int cmd);

static int iansOpenReceive (struct sk_buff *skb, struct net_device *dev, struct packet_type *pt);

static struct net_device *iansInitEtherdev( const char *name );

static struct packet_type iansPacketType =
{
	IANS_FRAME_TYPE,                        // Note : no need for htons !!!
        NULL,                                   // Null = All devices 
	iansOpenReceive, // was iansReceive,  // receive function
        NULL,                                   // Private to the packet type          
        NULL,                                   // next
};


//--------------------- CONVERT OS TYPES TO ANS TYPES -------------------------


static inline int _convertOsMCListToAnsMCList(struct dev_mc_list *osList,
                                int osCount,
                                MULTICAST_LIST *ansList)
{

    struct dev_mc_list *currList;
    int res;
    
    ASSERT(osList && ansList);

    for(currList = osList; currList != NULL; currList = currList->next) {
        res = ANSAddMulticastAddress (ansList,
                    (MAC_ADDR*)currList->dmi_addr,
                    currList->dmi_addrlen,
                    currList->dmi_users,
					currList->dmi_gusers,
                    NULL);

        if (res != FLOW_OK) {
	  if (res == FLOW_GP_RESOURCES){
	    IANS_ERROR("kmalloc failed.\n");
	  }
	  IANS_ERROR("Failed in adding a multicast address.\n");
	  return IANS_STAM;
        }
    }

    ASSERT(osCount == ansList->NumAddresses);

    return IANS_OK;
}


static inline void _convertOsPFToAnsPF(unsigned short osFlags, PACKET_FILTER *packetFilter){

    if(osFlags & IFF_PROMISC) {
        *packetFilter |= PF_PROMISCUOUS_BIT;
    }
    else {
        *packetFilter &= ~(PF_PROMISCUOUS_BIT);
    }

    if(osFlags & IFF_ALLMULTI) {
        *packetFilter |= PF_ALLMCA_BIT;
    }
    else {
        *packetFilter &= ~(PF_ALLMCA_BIT);
    }
}


static inline void _convertOsMessageToAnsMessage(pOsMessage_t osMsg, message_t *ansMsg, IANS_FLOW_MODE flow) {

 
  ASSERT(osMsg && ansMsg);
  
  ansMsg->pOsMessage = osMsg;
  ansMsg->pProtocol = &(osMsg->protocol);  
  ansMsg->Head = osMsg->head;                 
  ansMsg->Data = osMsg->data;
  //We keep the attributes during
  //transmit in the cb[] buffer. The base driver puts
  //its atributes at the head of the buffer.
  //Eventualy the receive should use the same buffer as the transmit.
  if (flow == TRANSMIT_FLOW)
    ansMsg->AttrHeader = osMsg->cb;
  else
    ansMsg->AttrHeader = osMsg->head;
  
  ansMsg->LocalAttr = NULL;
  ansMsg->Len = osMsg->tail - osMsg->data;      
  ansMsg->pDest = DEST_ADDR_PTR(osMsg, flow);
  ansMsg->pSrc = SRC_ADDR_PTR(osMsg, flow); 
  ansMsg->pLengthType = LENGTH_TYPE_PTR(osMsg, flow);

}



//------- (OS/ANS) NET DEVICE ALLOCATION AND INITIALIZATION FUNCTIONS --------


//Alocates a kernel's struct device for a vadapter
struct net_device *iansDevAlloc( const char *name ) {
       
        int allocSize;
	struct net_device *dev;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) 
	struct net_device *temp;
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	if ( dev_get_by_name( name) != NULL ) {
#else
	if ( (temp = dev_get_by_name(name)) != NULL ) {
	  dev_put(temp);
#endif
	  IANS_ERROR("A device with this name (%s) already exists\n", name);
	  return NULL;
	}

	//For kernel 2.4.0 up dev->name is defined as char name[IFNAMSIZ].
	//For kernel 2.4.0 down dev->name is defined as char *name. We allocate
	//extra space to store the name right after dev.
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	allocSize = sizeof(struct net_device)+IFNAMSIZ;
#else
	allocSize = sizeof(struct net_device);
#endif	
	allocSize += 3;	
	allocSize &= ~3;

	iNCGGPSizedMemAlloc ((void **)(&dev) , allocSize);
	if (dev == NULL) {
	  IANS_ERROR ("kmalloc failed.\n");
	  return NULL;
	}
	
	memset( dev , 0 , allocSize );

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	dev->name=(char *)(dev+1);	// Name string space
#endif

	iAnsStrCpy(dev->name,name);

	return dev;
}


static struct net_device *iansInitEtherdev( const char *name ) {
  
	struct net_device *dev;

	if ( (  dev = iansDevAlloc ( name ) ) == NULL )
		return NULL;

	ether_setup(dev); 	
	
	return dev;
}


// Allocates and initializes an OS struct device for a vadapter.
// Creates the ANS device structure for the vadapter.
int iansInitVadapterDev(const char *name, device_t **ansVadapterDev) {
  struct net_device  *osVadapterDev;
	
  ASSERT(name);
 
  if ( ( osVadapterDev = iansInitEtherdev( name ) ) == NULL ) {
    return IANS_VADAPTER_INITIALIZE_FAIL;
  }

  //Allocate Ans vadapter device structure
  iNCGGPSizedMemAlloc ((void **)(ansVadapterDev), sizeof(device_t) ); 
    if (*ansVadapterDev == NULL) {
      IANS_ERROR("Failed to allocate an IANS device structure\n");
      iNCGGPMemFree( (void **) (&osVadapterDev) );
      return IANS_MEM_ERROR;
    }
  
  // The ANS-specific entries in the device structure.
  osVadapterDev->open               =  iansVadapterOpen;
  osVadapterDev->hard_start_xmit    =  iansVadapterSend;
  osVadapterDev->stop               =  iansVadapterStop;
  osVadapterDev->get_stats          =  iansVadapterGetStats;
  osVadapterDev->set_multicast_list =  iansVadapterSetMulticastList;
  osVadapterDev->change_mtu         =  iansVadapterChangeMtu;
  osVadapterDev->set_mac_address    =  iansVadapterSetMacAddress;
  osVadapterDev->do_ioctl           =  iansVadapterDoIoctl;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
  osVadapterDev->start              =  0;
#endif
  
  //Initialize some os dependant fields in the Ans device structure.     
  (*ansVadapterDev)->pOsDev = osVadapterDev;   
  (*ansVadapterDev)->name = osVadapterDev->name;
  (*ansVadapterDev)->mtu = &(osVadapterDev->mtu);
  (*ansVadapterDev)->mac = (MAC_ADDR *)(osVadapterDev->dev_addr);
 
   return IANS_OK;
}


// Creates the ANS subtitude for the kernel's struct device
// for an adapter.
int iansInitMemberDev(const char *baseName, device_t **ansDev){

  struct net_device *dev;
  
  if (  (dev = dev_get_by_name( baseName ) ) == NULL ) {
	IANS_ERROR("Device %s doesn't exist!.\n", baseName);
	return 	IANS_MEMBER_NAME_WRONG;
  }



  //create the related Ans device.
  iNCGGPSizedMemAlloc ((void **)(ansDev), sizeof(device_t));
  if ( *ansDev == NULL ) {
    IANS_ERROR("Failed to allocate an IANS device structure for a member.\n");
    return IANS_MEM_ERROR;
  }

  
  (*ansDev)->pOsDev = dev;   
  (*ansDev)->name = dev->name;
  (*ansDev)->tbusy = FALSE;
  (*ansDev)->mac = (MAC_ADDR *)(dev->dev_addr);
  (*ansDev)->mtu = &(dev->mtu);
  (*ansDev)->open = baseOpenDevice;
  (*ansDev)->close = baseCloseDevice;
  (*ansDev)->send = (int (*) (void *, void *) ) baseSend;
  (*ansDev)->doIoctl = baseDoIoctl; 
  (*ansDev)->setMacAddr = baseSetMacAddr;
  (*ansDev)->getStats = baseGetStats;
  (*ansDev)->setMulticastList = baseSetMulticastList;
  (*ansDev)->changeMtu = baseChangeMtu;
	
  return IANS_OK;
}


int iansRegisterDev(device_t *ansDev) {

	struct net_device * dev;

	ASSERT(ansDev);
	ASSERT(ansDev->name);

	dev = (struct net_device *) GET_OS_DEV(ansDev);

	if(dev == NULL) {
		return IANS_VADAPTER_REGISTER_FAIL;
	}

	register_netdevice(dev);
	return IANS_OK;
}

//********************************************************************************
//void iansUnregisterDev(device_t *ansDev,BOOLEAN close, BOOLEAN ioctlContext )
//********************************************************************************
//Unregisters net_devices.
//The kernel registers and unregisters netdevices from within a lock (rtnl_lock)
//Getting devices from the kernel device list uses a read_lock.

//The kernel function unregister_netdev(dev_name) locks the rtnl_lock and then
//calls unregister_netdevice(dev_name) from within the lock. 

//The kernel function dev_get(dev_name) locks the read lock and calls
// dev_get_by_name(dev_name) from within the lock.

//When calling iansUnregisterDev() from ioctl context we are allready locked by
//an rtnl_lock, which is locked by the kernel before execution of ioctls, in that case
//we must use unregister_netdevice(dev_name) and dev_get_by_name(dev_name) to avoid a
//a dead lock.
//********************************************************************************** 
 void iansUnregisterDev(device_t *ansDev,BOOLEAN close) {

	struct net_device * devExist = NULL;
	
        BOOLEAN ioctlContext = ANSGetConfigExecuteCommand();
	
	ASSERT(ansDev);
	ASSERT(ansDev->name);

	if( (devExist = dev_get_by_name(ansDev->name)) ) {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) 
	  dev_put(devExist);
#endif
	  if (devExist != (struct net_device *) GET_OS_DEV(ansDev)) {
	    IANS_ERROR("Mismatch of kernel and ANS pointers to %s's device structure - cann't unregister the net device!\n", ansDev->name);
	    return;
	  }
	  else {//unregister the device.
	    if (close) {
	      dev_close ( devExist );
	    }

	    (ioctlContext ? unregister_netdevice(devExist) : unregister_netdev(devExist));

	  }
	}
	else {
	  IANS_ERROR("Device %s doesn't exist - cann't unregister the net device!\n", ansDev->name);
	}
}



// ----------------------CONTROL NET DEVICE FUNCTIONS --------------------------
 
//  Here we accept the control IOCTLS. 
static int iansControlDoIoctl ( struct net_device *dev, struct ifreq *rq, int cmd)
{
  int res;

  char *data;

  data = rq->ifr_ifru.ifru_data;

  if ( cmd != SIOCDEVPRIVATE ) {
      IANS_DEBUG("Wrong Ioctl\n");
      return IANS_OS_ERROR;
  } 

  if ( dev != control_g ) {
    IANS_DEBUG("Got control IOCTL on non control device : dev->name %s\n",dev->name);
    return IANS_OS_ERROR;
  }

  if ( data == NULL ) {
    IANS_DEBUG("Got null data in control IOCTL\n");
    return IANS_OS_ERROR;
  }

  ANSSetConfigExecuteCommand(1);
  res=ANSDoConfigIoctl( data );
  ANSSetConfigExecuteCommand(0);
  
  return res;
}

 
static int  iansControlOpen( struct net_device *dev ) {
  IANS_DEBUG ("Got open on control device\n");
  return 0;
}

static int  iansControlStop( struct net_device *dev ) {
  IANS_DEBUG ("Got stop on control device\n");
  return 0;
}
 
static struct net_device_stats *iansControlGetStats(struct net_device *dev ) {
//	IANS_DEBUG ("Got get_statistics for control device\n");
	return NULL;
}

static void  iansControlSetMCList( struct net_device *dev ) {
	IANS_DEBUG ("Got set_multicast_list for control device\n");
}

static int  iansControlSend(struct sk_buff *skb, struct net_device *dev) {
	IANS_DEBUG ("Got hard_start_xmit for control device\n");
	return 0;
}



// ---------------------- VADAPTER OS NET DEVICE FUNCTIONS ------------------------


static int iansVadapterOpen(struct net_device *dev) {

    device_t *ansDev = NULL;
    int res = 0;
    int ansRes = IANS_OK;

    iansStartBH(&ians_lock);

    ansDev = ANSGetVadapterAnsDevice(dev->name);
    ASSERT(ansDev);

    ansRes = ansDev->open(ansDev);
    if (ansRes == IANS_OK){      
      MOD_INC_USE_COUNT;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
      dev->start = 1;
#endif      
    }
    else{
      res = -EPERM; //operation not permitted 
    }
    
    iansEndBH(&ians_lock);

    return res;

    
}


static  int iansVadapterStop ( struct net_device *dev )
{

    device_t *ansDev = NULL; 

    iansStartBH(&ians_lock);

    MOD_DEC_USE_COUNT;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
    dev->start = 0;
#endif

    ansDev = ANSGetVadapterAnsDevice(dev->name);
    ASSERT(ansDev);

    ansDev->close(ansDev);

    iansEndBH(&ians_lock);

    return 0;
}


static struct net_device_stats *iansVadapterGetStats ( struct net_device *dev )
{
  device_t *ansDev = NULL;
  struct net_device_stats *res = NULL;

  ASSERT(dev);

  iansStartBH(&ians_lock);

  ansDev = ANSGetVadapterAnsDevice(dev->name);
  ASSERT(ansDev);

  res =(struct net_device_stats *) ansDev->getStats(ansDev);

  iansEndBH(&ians_lock);

  return res;      
} 


// This function handles both the packet filter and the multicast list.
static void  iansVadapterSetMulticastList ( struct net_device *dev )
{
  MULTICAST_LIST multicastList;
  PACKET_FILTER  packetFilter;
  device_t *ansDev = NULL;
  int res;

    ASSERT(dev);

    iansStartBH(&ians_lock);

    memset( &multicastList , 0 , sizeof(MULTICAST_LIST) );


    if(dev->mc_list){
        ASSERT(dev->mc_count);
        res = _convertOsMCListToAnsMCList(dev->mc_list, dev->mc_count, &multicastList);
        if(res != IANS_OK) {
             iansEndBH(&ians_lock);
	     return;
        }
    }

    _convertOsPFToAnsPF(dev->flags, &packetFilter);

    ansDev = ANSGetVadapterAnsDevice(dev->name);
    ASSERT(ansDev);

    res = ansDev->setMulticastList(ansDev, &multicastList, packetFilter);
    if(res != IANS_OK) {
            IANS_ERROR("Set multicast list failed!\n");
            iansEndBH(&ians_lock);
            return;
    }
    
    iansEndBH(&ians_lock);
}


static int iansVadapterChangeMtu( struct net_device *dev, int new_mtu)
{
        device_t *ansDev = NULL;
        int res;

        ASSERT(dev);

        if(new_mtu == dev->mtu){
                //nothing to do
                return 0;
        }

        if(new_mtu < 0) {
                //must be positive
                return -EINVAL;
        }

        iansStartBH(&ians_lock);

        ansDev = ANSGetVadapterAnsDevice(dev->name);

        ASSERT(ansDev);

        res = ansDev->changeMtu(ansDev, new_mtu);
        if(res != IANS_OK) {
                IANS_ERROR("Change MTU failed!\n");
                iansEndBH(&ians_lock);
                return -EINVAL;
        }

        iansEndBH(&ians_lock);
        return 0;
}



static int iansVadapterSetMacAddress(struct net_device *dev, void *addr)
{
        device_t *ansDev = NULL;
        sockAddr_t *sockaddr;
        int res;
        
        ASSERT(dev && addr);

        iansStartBH(&ians_lock);

        ansDev = ANSGetVadapterAnsDevice(dev->name);

        ASSERT(ansDev);

        sockaddr = (sockAddr_t*)addr;
        
        res = ansDev->setMacAddr(ansDev, sockaddr);
        if(res != IANS_OK) {
                IANS_ERROR("Set MAC address failed!\n");
                iansEndBH(&ians_lock);
                return -1;
        }

        iansEndBH(&ians_lock);
        return 0;
}



static int iansVadapterDoIoctl ( struct net_device *dev, struct ifreq *rq, int cmd) {
	device_t *ansDev = NULL;
	int res;

	ASSERT(dev);	

	iansStartBH(&ians_lock);

	if ( cmd == SIOCDEVPRIVATE ) { // should not get private ioctls here.  
	  iansEndBH(&ians_lock);
	  return 0;
	}

	ansDev = ANSGetVadapterAnsDevice(dev->name);
	ASSERT(ansDev);

	res = ansDev->doIoctl(ansDev, rq, cmd);

	iansEndBH(&ians_lock);

	return res;
}



static int iansVadapterSend (struct sk_buff *skb, struct net_device *dev) {
  device_t *ansDev;
  static message_t msg;  //automatic variable to save allocation time. 
  int res = 0;
  int ansRes = IANS_OK;

  ASSERT(skb && dev);
  
  ASSERT(dev->name);

  iansSpinLock(&ians_lock);
  
  ansDev = ANSGetVadapterAnsDevice(dev->name);

  ASSERT(ansDev);
  ASSERT(((struct net_device *) GET_OS_DEV(ansDev)) == dev) 
  
  _convertOsMessageToAnsMessage(skb, &msg, TRANSMIT_FLOW);

  ansRes = ansDev->send(&msg, ansDev);
  if (ansRes != IANS_OK){
        res = -ENETDOWN;
  }
  
  iansSpinUnlock(&ians_lock);

  return res;

}



//--------------------------- WRAPPER FUNCTIONS ------------------------------
//returns kernels ticks per second
int iansGetKernelFreq(void) {

  return HZ;

}


void iansDevPut(device_t *ansDev) {
      

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
  struct net_device *dev;
  ASSERT(ansDev);
  dev = (struct net_device *) GET_OS_DEV(ansDev);
  ASSERT(dev);
  dev_put(dev);	
#endif
  
}


BOOLEAN iansBaseDevIsOpen(device_t *ansDev) {
      struct net_device *dev=NULL;
      ASSERT(ansDev);
      dev = (struct net_device *) GET_OS_DEV(ansDev);
      ASSERT(dev);

      if ( netif_running(dev) || (dev->flags & IFF_UP))
	    return TRUE;
      return FALSE;
}


unsigned long iansGetDeviceBaseAddr(device_t *ansDev) {
      struct net_device *dev=NULL;
      ASSERT(ansDev);
      dev = (struct net_device *) GET_OS_DEV(ansDev);
      ASSERT(dev);     

      return dev->base_addr;
}


int iansIfReceive(device_t *ansDev, message_t *msg) {
      struct sk_buff *skb;
      int FlowStatus = FLOW_DOESNT_EXIST;
      
      ASSERT(ansDev && msg);
      skb = (struct sk_buff *) GET_OS_MSG(msg);
      ASSERT(skb);

      // set to vAdapter device
      skb->dev = (struct net_device *) GET_OS_DEV(ansDev);

      ASSERT(skb->dev);
         
      netif_rx (skb);	  // pass the packet up the stack 
      
      FlowStatus = FLOW_TRANSFERED;
            
      return ( FlowStatus );

}


void iansIfStop(device_t *ansDev) {

  netif_stop_queue((struct net_device *) GET_OS_DEV(ansDev));

}


void iansIfWake(device_t *ansDev) {

 netif_wake_queue((struct net_device *) GET_OS_DEV(ansDev));

}


void  iansConvertProbeMessageToAnsMessage(pOsMessage_t osMsg, 
					  message_t *ansMsg, IANS_FLOW_MODE flow) {

  _convertOsMessageToAnsMessage(osMsg, ansMsg, flow);

}


int iansGetSizeofStats(void){
  return (int)sizeof(struct net_device_stats);
}


void iansAddStats(pstats_t destStats, pstats_t srcStats) {

  ASSERT(destStats && srcStats);

  //  destStats->rx_packets += srcStats->rx_packets;
  //  destStats->tx_packets += srcStats->tx_packets;
  destStats->rx_bytes += srcStats->rx_bytes;
  destStats->tx_bytes += srcStats->tx_bytes;
  destStats->rx_errors += srcStats->rx_errors;
  destStats->tx_errors += srcStats->tx_errors;
  destStats->rx_dropped += srcStats->rx_dropped;
  destStats->tx_dropped += srcStats->tx_dropped;
  destStats->multicast += srcStats->multicast;
  destStats->collisions += srcStats->collisions; 
  destStats->rx_length_errors += srcStats->rx_length_errors;
  destStats->rx_over_errors += srcStats->rx_over_errors;
  destStats->rx_crc_errors += srcStats->rx_crc_errors;
  destStats->rx_frame_errors += srcStats->rx_frame_errors;
  destStats->rx_fifo_errors += srcStats->rx_fifo_errors;
  destStats->rx_missed_errors += srcStats->rx_missed_errors;

  /* detailed tx_errors */
  destStats->tx_aborted_errors += srcStats->tx_aborted_errors;
  destStats->tx_carrier_errors += srcStats->tx_carrier_errors;
  destStats->tx_fifo_errors += srcStats->tx_fifo_errors;
  destStats->tx_heartbeat_errors += srcStats->tx_heartbeat_errors;
  destStats->tx_window_errors += srcStats->tx_window_errors;
	

  /* for cslip etc */
  destStats->rx_compressed += srcStats->rx_compressed;
  destStats->tx_compressed += srcStats->tx_compressed;
 
}


void iansSubStats(pstats_t destStats, pstats_t srcStats) {

  ASSERT(destStats && srcStats);

  // destStats->rx_packets -= srcStats->rx_packets;
  // destStats->tx_packets -= srcStats->tx_packets;
  destStats->rx_bytes -= srcStats->rx_bytes;
  destStats->tx_bytes -= srcStats->tx_bytes;
  destStats->rx_errors -= srcStats->rx_errors;
  destStats->tx_errors -= srcStats->tx_errors;
  destStats->rx_dropped -= srcStats->rx_dropped;
  destStats->tx_dropped -= srcStats->tx_dropped;
  destStats->multicast -= srcStats->multicast;
  destStats->collisions -= srcStats->collisions; 
  destStats->rx_length_errors -= srcStats->rx_length_errors;
  destStats->rx_over_errors -= srcStats->rx_over_errors;
  destStats->rx_crc_errors -= srcStats->rx_crc_errors;
  destStats->rx_frame_errors -= srcStats->rx_frame_errors;
  destStats->rx_fifo_errors -= srcStats->rx_fifo_errors;
  destStats->rx_missed_errors -= srcStats->rx_missed_errors;

  /* detailed tx_errors */
  destStats->tx_aborted_errors -= srcStats->tx_aborted_errors;
  destStats->tx_carrier_errors -= srcStats->tx_carrier_errors;
  destStats->tx_fifo_errors -= srcStats->tx_fifo_errors;
  destStats->tx_heartbeat_errors -= srcStats->tx_heartbeat_errors;
  destStats->tx_window_errors -= srcStats->tx_window_errors;
	

  /* for cslip etc */
  destStats->rx_compressed -= srcStats->rx_compressed;
  destStats->tx_compressed -= srcStats->tx_compressed;
 
}


void iansResetStats(pstats_t stats) {

  ASSERT(stats);
  memset((void *)(stats), 0, sizeof(struct net_device_stats));  

}


void iansSetStats(pstats_t stats, ULONG rx_packets, ULONG tx_packets)
{
    stats->rx_packets = rx_packets;
    stats->tx_packets = tx_packets;
}


void iansLock(void) {
    iansSpinLock(&ians_lock);
}


void iansUnlock(void) {
    iansSpinUnlock(&ians_lock);
}


void iansLockBH(void) {
    iansStartBH(&ians_lock);
}


void iansUnlockBH(void) {
    iansEndBH(&ians_lock);
}

void iansHotLock(BOOLEAN blockIntr, int *flags) {
  if(blockIntr){
    iansStartIrq(&ians_ind_lock, (*flags));
  }
  else {
    iansSpinLock(&ians_ind_lock);
  }
}

void iansHotUnlock(BOOLEAN blockIntr, int *flags) {
  if(blockIntr){
    iansEndIrq(&ians_ind_lock, (*flags));
  }
  else {
    iansSpinUnlock(&ians_ind_lock);
  }         
}

BOOLEAN iansIsKernelVersionGreaterThan2_4_0(void){
  return KERNEL_VERSION_GREATER_THAN_2_4_0;
}

unsigned short iansUsecsToTicks (unsigned long microsecs){

	unsigned short ticks;
	ticks=microsecs*0.000001*HZ;
	return ticks;
}

//--------------------------------- MODULE FUNCTIONS ---------------------------

int ians_init_module(void) {
        
        int res=0;
	
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	KERNEL_VERSION_GREATER_THAN_2_4_0 = TRUE;
#else
	KERNEL_VERSION_GREATER_THAN_2_4_0 = FALSE;
#endif
	iansSpinLockInit(&ians_lock);
	iansSpinLockInit(&ians_ind_lock);
	
	res = ANSConfigCreate();
	if (res!=IANS_OK)
		return -1;

	if ( ( control_g = iansInitEtherdev( "ians" ) ) == NULL ) {
		IANS_ERROR ("Init etherdev failed - cannot load the IANS module.\n");
		return -1;
	}

	control_g->open               =  iansControlOpen;
	control_g->hard_start_xmit    =  iansControlSend;
	control_g->stop               =  iansControlStop;
	control_g->get_stats          =  iansControlGetStats;
	control_g->set_multicast_list =  iansControlSetMCList;
	control_g->do_ioctl           =  iansControlDoIoctl;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	control_g->start              =  0;
#endif

	register_netdev(control_g);

	dev_add_pack ( &iansPacketType );
	
	return 0;
}


void ians_cleanup_module(void) {
  
	int res;
 
	dev_remove_pack( & iansPacketType );

	unregister_netdev( control_g );

       	iNCGGPMemFree( (void**)(&control_g) );

	ANSUnregisterAllVadapters(TRUE /*force*/);

	ANSStopActivity();

	//remove all the topology.
	iansStartBH(&ians_lock);
	res = ANSRemoveAll();
	if (res != IANS_OK) {
		IANS_DEBUG("Failed to remove IANS topology\n");
	}
	iansEndBH(&ians_lock);
	

#ifdef MEM_DEBUG
	if(iNCGGPMemGetAllocCount()) {
	        IANS_DEBUG("alloc_count = %d\n", alloc_count);
	}
#endif
	
}


static int iansOpenReceive (struct sk_buff *skb, struct net_device *dev, struct packet_type *pt)
{
	int status = 0;
	message_t msg;

	ASSERT(skb && dev);
       
	iansSpinLock(&ians_lock);

	if (skb->protocol != IANS_FRAME_TYPE) {
		IANS_ERROR("Protocol of received sk_buff differs from the IANS frame type!\n");
		IANS_ERROR("Protocol is set to %x\n", skb->protocol);
		status = 0;
   	}
	else {
	        _convertOsMessageToAnsMessage(skb, &msg, RECEIVE_FLOW); 
	        status = ANSReceive(&msg, dev->name,(pPacket_t) pt);
	}

	if (status == 1)
	  {
	    //The packet has the ANS protocol but the 
      	    //adapter sending up this packet is not a member.
	    //This is either a probe packet or some other packet
	    //that we don't recognize.	    
	    // drop it.
	    dev_kfree_skb (skb);
	    status = 0;
	  }

	status = 0;
 
	iansSpinUnlock(&ians_lock);

	return status;
}


module_init(ians_init_module);
module_exit(ians_cleanup_module);
